#
# spec file for package cri-o
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


#Compat macro for new _fillupdir macro introduced in Nov 2017
%if ! %{defined _fillupdir}
  %define _fillupdir /var/adm/fillup-templates
%endif

%define project github.com/kubernetes-incubator/cri-o
# Build with libostree-devel in Tumbleweed, Leap 15+ and SLES 15+
%if 0%{?suse_version} >= 1500
%define with_libostree 1
%endif
# Define macros for further referenced sources
%define	name_source1 crio.service
%define	name_source2 sysconfig.crio
%define	name_source3 crio.conf
Name:           cri-o
Version:        1.9.10
Release:        19.1
Summary:        OCI-based implementation of Kubernetes Container Runtime Interface
License:        Apache-2.0
Group:          System/Management
Url:            https://github.com/kubernetes-incubator/cri-o
Source0:        %{name}-%{version}.tar.xz
Source1:        %{name_source1}
Source2:        %{name_source2}
Source3:        %{name_source3}
Source4:        cri-o-rpmlintrc
BuildRequires:  device-mapper-devel
BuildRequires:  fdupes
BuildRequires:  git-core
BuildRequires:  glib2-devel-static
BuildRequires:  glibc-devel-static
BuildRequires:  go-go-md2man
BuildRequires:  golang-packaging
BuildRequires:  libapparmor-devel
BuildRequires:  libassuan-devel
BuildRequires:  libbtrfs-devel
BuildRequires:  libgpgme-devel
BuildRequires:  libseccomp-devel
BuildRequires:  golang(API) >= 1.8
Requires:       cni
Requires:       cni-plugins
Requires:       iproute2
Requires:       iptables
Requires:       libcontainers-common
Requires:       libcontainers-image
Requires:       libcontainers-storage
Requires:       runc >= 1.0.0~rc4
Requires:       socat
# disable stripping of binaries
%{go_nostrip}
%if 0%{?with_libostree}
BuildRequires:  libostree-devel
%endif

%description
CRI-O provides an integration path between OCI conformant runtimes
and the kubelet. Specifically, it implements the Kubelet Container Runtime
Interface (CRI) using OCI conformant runtimes. The scope of CRI-O is tied to
the scope of the CRI.

%prep
%setup -q

%build
# We can't use symlinks here because go-list gets confused by symlinks, so we
# have to copy the source to $HOME/go and then use that as the GOPATH.
export GOPATH=$HOME/go
mkdir -pv $HOME/go/src/%{project}
rm -rf $HOME/go/src/%{project}/*
cp -avr * $HOME/go/src/%{project}
cd $HOME/go/src/%{project}

export BUILDTAGS="seccomp apparmor containers_image_ostree_stub"
%if 0%{?with_libostree}
export BUILDTAGS="seccomp apparmor"
echo "Compiling with libostree support"
%endif

# Build crio
go build -tags "$BUILDTAGS" \
         -o bin/crio \
         -buildmode=pie \
         %{project}/cmd/crio

# Build conmon
make conmon

# Build pause
make pause

# Build manpages
make %{?_smp_mflags} docs

%check
cd $HOME/go/src/%{project}
export GOPATH=$HOME/go
export BUILDTAGS="seccomp apparmor containers_image_ostree_stub"
%if 0%{?with_libostree}
export BUILDTAGS="seccomp apparmor"
%endif

PKG_LIST=$(go list -tags "${BUILDTAGS}" ./... | \
           grep -v %{project}/vendor} | \
           grep -v %{project}/server)
go test -buildmode=pie -tags "$BUILDTAGS" $PKG_LIST

%pre
%service_add_pre %{name_source1}

%post
%service_add_post %{name_source1}

%preun
%service_del_preun %{name_source1}

%postun
%service_del_postun %{name_source1}

%install
cd $HOME/go/src/%{project}

# Binaries
install -D -m 0755 bin/crio    %{buildroot}/%{_bindir}/crio
install -d %{buildroot}/%{_libexecdir}/crio/bin
install -D -m 0755 bin/conmon  %{buildroot}/%{_libexecdir}/crio/bin/conmon
install -D -m 0755 bin/pause   %{buildroot}/%{_libexecdir}/crio/bin/pause
# Manpages
install -d %{buildroot}/%{_mandir}/man5
install -d %{buildroot}/%{_mandir}/man8
install -m 0644 docs/crio.conf.5 %{buildroot}/%{_mandir}/man5
install -m 0644 docs/crio.8      %{buildroot}/%{_mandir}/man8
# Configs
install -D -m 0644 %{SOURCE3}       %{buildroot}/%{_sysconfdir}/crio/%{name_source3}
install -D -m 0644 seccomp.json     %{buildroot}/%{_sysconfdir}/crio/seccomp.json
install -D -m 0644 crio-umount.conf %{buildroot}/%{_datadir}/oci-umount/oci-umount.d/cri-umount.conf
install -D -m 0644 crictl.yaml      %{buildroot}/%{_sysconfdir}/crictl.yaml
install -D -m 0644 %{SOURCE2}       %{buildroot}%{_fillupdir}/%{name_source2}
# Systemd
install -D -m 0644 %{SOURCE1} %{buildroot}%{_unitdir}/%{name_source1}
# Symlinks to rc files
install -d -m 0755 %{buildroot}%{_sbindir}
ln -sf service %{buildroot}%{_sbindir}/rccrio

%fdupes %{buildroot}/%{_prefix}

%files
# Binaries
%{_bindir}/crio
%dir %{_libexecdir}/crio
%dir %{_libexecdir}/crio/bin
%{_libexecdir}/crio/bin/conmon
%{_libexecdir}/crio/bin/pause
# Manpages
%{_mandir}/man5/crio.conf.5*
%{_mandir}/man8/crio.8*
# Configs
%dir %{_sysconfdir}/crio
%config(noreplace) %{_sysconfdir}/crio/%{name_source3}
%config %{_sysconfdir}/crio/seccomp.json
%dir %{_datadir}/oci-umount
%dir %{_datadir}/oci-umount/oci-umount.d
%{_datadir}/oci-umount/oci-umount.d/cri-umount.conf
%config(noreplace) %{_sysconfdir}/crictl.yaml
%{_fillupdir}/%{name_source2}
# Systemd
%{_unitdir}/%{name_source1}
%{_sbindir}/rccrio

%changelog
* Thu Mar 15 2018 vrothberg@suse.com
- Require cni and cni-plugins to enable container networking.
  feature#crio
* Thu Mar 15 2018 vrothberg@suse.com
- Update cri-o to v1.9.10:
  * conmon: Avoid strlen in logging path
  * conmon: Remove info logs
  * container_exec: Fix terminal setting for exec
* Mon Mar 12 2018 vrothberg@suse.com
- Update cri-o to v1.9.9:
  * sandbox_stop: Call CNI stop before stopping pod infra container
* Thu Mar  8 2018 vrothberg@suse.com
- Remove the crio-shutdown.service.  It does not have any effect when
  shutting down crio and also isn't shipped on Fedora.
  - crio-shutdown.service
* Mon Mar  5 2018 vrothberg@suse.com
- crio.conf: update default socket to /var/run/crio/crio.sock as suggested
  by upstream.
* Mon Mar  5 2018 vrothberg@suse.com
- Update cri-o to v1.9.8:
  * system_containers: Update mounts
  * execsync: Set terminal to true when we pass -t to conmon
  * Make network namespace pinning optional
  * Add context to net ns symlink removal errors
  * Make the /opt/cni mount rw
  * sandbox_stop: close/remove the netns _after_ stopping the containers
  * sandbox net: set netns closed after actaully closing it
* Mon Mar  5 2018 vrothberg@suse.com
- Configuration files should generally be tagged as %%config(noreplace) in order
  to keep the modified config files and to avoid losing data when the package
  is being updated.
* Sat Mar  3 2018 vrothberg@suse.com
- Remove empty filter rule from cri-o-rpmlintrc, which was mistakenly
  masking a few warnings, some of which have been fixed, others need
  to be filtered.  conmon and pause are not compiled with -fpie anymore
  to align with what upstream does; linking fails when done properly.
* Fri Mar  2 2018 fcastelli@suse.com
- Update minimum version of the Go compiler required
* Fri Mar  2 2018 fcastelli@suse.com
- Add missing runtime dependencies: socat, iptables, iproute
* Wed Feb 28 2018 vrothberg@suse.com
- Change the installation path of conmon and pause from
  /usr/lib/crio to /usr/lib/crio/bin in order to align with upstream
  requirements.
- Update crio.conf to the reflect the new path of conmon and set the correct
  path of CNI plugins (i.e., /usr/lib/cni).
* Tue Feb 20 2018 vrothberg@suse.com
- Update cri-o to v1.9.6:
  * vendor: update c/image to handle text/plain from registries
    Fixes cases where text/plain s1 schemes are mistakenly converted
    to MIME.
* Sun Feb 18 2018 jengelh@inai.de
- Let description say what the package really does.
* Fri Feb 16 2018 vrothberg@suse.com
- Update cri-o to v1.9.5:
  * system container: add /var/tmp as RW
  * container_create: correctly set user
  * imageService: cache information about images
  * image: Add lock around image cache access
* Fri Feb 16 2018 vrothberg@suse.com
- Cleanup version-update related changelogs to only keep log entries of
  changes that are visible and important to the user, and the project.
* Mon Feb 12 2018 vrothberg@suse.com
- Add requirements to libcontainers-{common,image,storage}.
- Run spec-cleaner on cri-o.spec.
* Mon Feb 12 2018 vrothberg@suse.com
- Update cri-o to v1.9.3:
  * Be more diligent about cleaning up failed-to-create containers
  * Use crictl instead of crioctl in image integration tests
  * Handle truncated IDs in imageService.ResolveNames()
  * Switch to ImageServer.UntagImage in RemoveImage handler
  * Return image references from the storage package
  * storage: API fixups
* Fri Feb  9 2018 vrothberg@suse.com
- Use golang-packaging macro for binary stripping.
- Use -buildmode=pie for compilation.
- The update to 1.9.0+ removes the crioctl binary.  The crictl binary from
  cri-tools should be used instead.
- Update cri-o to v1.9.2:
  * sandbox: fix sandbox logPath when crio restarts
  * Adapt to recent containers/image API updates
  * container_create: only bind mount /etc/hosts if not provided by k8s
  * container_attach: Ensure ctl file is closed
  * lib,oci: drop stateLock when possible
  * container_exec: fix terminal true process json
  * container_create: fix apparmor from container config
  * container_create: correctly set image and kube envs
  * oci: do not append conmon env to container process
  * container_exec: use process file with runc exec
  * drop crioctl source code
  * conmon: Add support for partial/newline log tags
  * image_pull: fix image resolver
  * Add /proc/scsi to masked paths
  * replace crioctl with crictl
  * replace crioctl in e2e with crictl
  * Move crio default sock to /var/run/crio/crio.sock
  * container_create: set the seccomp profile in the container object
* Mon Feb  5 2018 vrothberg@suse.com
- Fix libostree-devel %%if condition for TW, Leap 15+ and SLES 15+.
* Thu Feb  1 2018 vrothberg@suse.com
- Use `%%fdupes %%buildroot/%%_prefix` since `fdupes %%buildroot` is not allowedv
  because you cannot make hardlinks between certain partitions.
* Wed Jan 31 2018 vrothberg@suse.com
- Source the cri-o-rpmlintrc the spec file.
* Tue Jan 30 2018 vrothberg@suse.com
- Add cri-o package: CRI-O is meant to provide an integration path between OCI
  conformant runtimes and the kubelet. Specifically, it implements the Kubelet
  Container Runtime Interface (CRI) using OCI conformant runtimes. The scope of
  CRI-O is tied to the scope of the CRI.
